from _Framework.ControlSurface import ControlSurface
from typing import Any

from application.CommandBus import CommandBus
from application.Container import Container
from application.ScriptDisconnectedEvent import ScriptDisconnectedEvent
from application.ScriptResetActivatedEvent import ScriptResetActivatedEvent
from application.command.ReloadScriptCommand import ReloadScriptCommand
from domain.shared.errors.ErrorRaisedEvent import ErrorRaisedEvent
from domain.shared.event.DomainEventBus import DomainEventBus
from shared.Song import Song
from shared.logging.Logger import Logger
from shared.sequence.Sequence import Sequence


class Protocol0(ControlSurface):
    def __init__(self, c_instance=None):
        # type: (Any) -> None
        super(Protocol0, self).__init__(c_instance=c_instance)
        self._initialize()

    def _initialize(self, reset=False):
        # type: (bool) -> None
        if reset:
            self.disconnect(reset)

        # noinspection PyBroadException
        try:
            Container(self)
        except Exception as e:
            print(e)
            DomainEventBus.emit(ErrorRaisedEvent())
            return

        DomainEventBus.subscribe(ScriptResetActivatedEvent, lambda _: self._initialize(reset=True))
        CommandBus.dispatch(ReloadScriptCommand())

        Logger.info("P0 script loaded")

    def disconnect(self, reset=False):
        # type: (bool) -> None
        if not reset:
            super(Protocol0, self).disconnect()

        DomainEventBus.emit(ScriptDisconnectedEvent())
        # without this, the events are going to be handled twice
        DomainEventBus.reset()
        Sequence.reset()

        for track in Song.all_simple_tracks():
            track.disconnect()
        for scene in Song.scenes():
            scene.disconnect()
