from types import MethodType

from _Framework.ControlSurface import ControlSurface, get_control_surfaces
from _Framework.Util import find_if
from application.Protocol0 import Protocol0
from domain.shared.event.DomainEventBus import DomainEventBus
from infra.logging.LoggerService import LoggerService
from infra.midi.MidiBytesReceivedEvent import MidiBytesReceivedEvent
from infra.midi.MidiBytesSentEvent import MidiBytesSentEvent
from shared.logging.LogLevelEnum import LogLevelEnum
from typing import Any, Tuple


class Protocol0Midi(ControlSurface):
    def __init__(self, c_instance=None):
        # type: (Any, bool) -> None
        # hide initializing message
        log_message = self.log_message
        self.log_message = lambda a: True
        super(Protocol0Midi, self).__init__(c_instance=c_instance)
        self.log_message = log_message
        # stop log duplication
        self._c_instance.log_message = MethodType(
            lambda s, message: None, self._c_instance
        )  # noqa
        self.main_p0_script = find_if(
            lambda s: isinstance(s, Protocol0), get_control_surfaces()
        )  # type: Protocol0

        self._logger = LoggerService()

        if self.main_p0_script is None:
            self._logger.log(
                "Error: couldn't find main Protocol0 script", level=LogLevelEnum.ERROR
            )
            return

        DomainEventBus.subscribe(MidiBytesSentEvent, self._on_midi_bytes_sent_event)

    def receive_midi(self, midi_bytes):
        # type: (Tuple) -> None
        DomainEventBus.emit(MidiBytesReceivedEvent(midi_bytes))

    def _on_midi_bytes_sent_event(self, event):
        # type: (MidiBytesSentEvent) -> None
        """Forward midi data to optional midi output ports (e.g. program change for Minitaur)"""
        self._send_midi(event.midi_bytes)
