from os.path import isfile, isdir

from typing import Optional

from domain.lom.device.Device import Device
from domain.lom.device.PluginDevice import PluginDevice
from domain.lom.device.RackDevice import RackDevice
from domain.lom.instrument.preset.preset_importer.DirectoryPresetImporter import (
    DirectoryPresetImporter,
)
from domain.lom.instrument.preset.preset_importer.FilePresetImporter import (
    FilePresetImporter,
)
from domain.lom.instrument.preset.preset_importer.NullPresetImporter import \
    NullPresetImporter
from domain.lom.instrument.preset.preset_importer.PluginDevicePresetImporter import (
    PluginDevicePresetImporter,
)
from domain.lom.instrument.preset.preset_importer.PresetImportInterface import (
    PresetImportInterface,
)
from domain.lom.instrument.preset.preset_importer.RackDevicePresetImporter import (
    RackDevicePresetImporter,
)
from domain.shared.errors.Protocol0Error import Protocol0Error


class PresetImporterFactory(object):
    @classmethod
    def create_importer(cls, device, preset_path, preset_extension):
        # type: (Optional[Device], str, str) -> PresetImportInterface
        if not preset_path or not preset_extension:
            return NullPresetImporter()
        elif isinstance(device, RackDevice):
            return RackDevicePresetImporter(device)
        elif isinstance(device, PluginDevice):
            return PluginDevicePresetImporter(device)
        elif isfile(preset_path):
            return FilePresetImporter(preset_path)
        elif isdir(preset_path):
            return DirectoryPresetImporter(preset_path, preset_extension)
        else:
            raise Protocol0Error("Couldn't import presets for %s" % device)
