from domain.lom.device.DeviceEnum import DeviceEnum
from domain.lom.track.group_track.ext_track.SimpleMidiExtTrack import SimpleMidiExtTrack
from domain.lom.track.routing.InputRoutingChannelEnum import InputRoutingChannelEnum
from domain.lom.track.routing.InputRoutingTypeEnum import InputRoutingTypeEnum
from domain.lom.track.routing.OutputRoutingTypeEnum import OutputRoutingTypeEnum
from domain.lom.validation.sub_validators.AggregateValidator import AggregateValidator
from domain.lom.validation.sub_validators.PropertyValueValidator import \
    PropertyValueValidator
from domain.lom.validation.sub_validators.SimpleTrackHasDeviceValidator import \
    SimpleTrackHasDeviceValidator
from domain.shared.BrowserServiceInterface import BrowserServiceInterface


class SimpleMidiExtTrackValidator(AggregateValidator):
    def __init__(self, track, browser_service):
        # type: (SimpleMidiExtTrack, BrowserServiceInterface) -> None
        self._track = track

        validators = [
            PropertyValueValidator(track, "volume", 0, name="midi track volume"),
            PropertyValueValidator(track, "muted", True, name="midi track muted"),
            SimpleTrackHasDeviceValidator(
                track, DeviceEnum.EXTERNAL_AUDIO_EFFECT, browser_service
            ),
            PropertyValueValidator(
                track.input_routing,
                "type",
                InputRoutingTypeEnum.ALL_INS,
                "midi track input type",
            ),
            PropertyValueValidator(
                track.input_routing,
                "channel",
                InputRoutingChannelEnum.CHANNEL_1,
                name="midi track input channel",
            ),
            PropertyValueValidator(
                track.output_routing,
                "type",
                OutputRoutingTypeEnum.SENDS_ONLY,
                name="midi track output type",
            ),
        ]

        super(SimpleMidiExtTrackValidator, self).__init__(validators)
