from typing import Optional

from protocol0 import EmptyModule
from application.Protocol0 import Protocol0
from application.control_surface.ActionGroupFactory import ActionGroupFactory
from domain.lom.set.AbletonSet import AbletonSet
from domain.lom.song.SongInitService import SongInitService
from domain.lom.track.group_track.matching_track.MatchingTrackInterface import MatchingTrackInterface
from domain.lom.track.routing.RoutingTrackDescriptor import RoutingTrackDescriptor
from domain.lom.track.simple_track.SimpleTrack import SimpleTrack
from domain.shared.backend.Backend import Backend
from domain.shared.scheduler.Scheduler import Scheduler
from domain.shared.utils.func import nop
from infra.logging.LoggerService import LoggerService
from infra.scheduler.BeatScheduler import BeatScheduler
from shared.UndoFacade import UndoFacade
from shared.logging.Logger import Logger
from tests.domain.fixtures.song import AbletonSong
from domain.lom.instrument.InstrumentDisplayService import InstrumentDisplayService
from tests.infra.scheduler.TickSchedulerTest import TickSchedulerTest


def make_protocol0():
    # type: () -> Protocol0
    live_song = AbletonSong()
    Protocol0.song = lambda _: live_song
    wait = Scheduler.wait
    Scheduler.wait = classmethod(nop)
    monkey_patch_static()
    p0 = Protocol0(EmptyModule(name="c_instance", is_false=False))
    monkey_patch_p0(live_song)
    Scheduler.wait = wait
    return p0


def monkey_patch_static():
    # hide logs
    Logger(LoggerService())
    Logger.dev = classmethod(nop)
    Logger.info = classmethod(nop)
    Logger.warning = classmethod(nop)

    Backend(nop)
    UndoFacade(nop, nop)
    AbletonSet.notify = nop
    SimpleTrack.click = nop
    # noinspection PyTypeChecker
    Scheduler(TickSchedulerTest(), None)  # ignore beat scheduling in tests

    # remove this until fixtures are thorough
    ActionGroupFactory.create_action_groups = classmethod(nop)

    InstrumentDisplayService._on_simple_track_armed_event = nop

    SongInitService.init_song = nop
    MatchingTrackInterface.disconnect = nop
    RoutingTrackDescriptor.__set__ = nop

    def log(_, message, *__, **___):
        print(message)

    Logger._log = classmethod(log)


def monkey_patch_p0(live_song=None):
    # type: (Optional[Live.Song.Song]) -> None
    Scheduler(TickSchedulerTest(), BeatScheduler(live_song))
